@@ -17,7 +17,7 @@ mod utils;
 mod cursor;
 mod event_queue;
 mod event_reader;
-mod len_and_epoch;
+mod chunk_state;
 #[allow(dead_code)]
 mod dynamic_array;
 
@@ -25,6 +25,9 @@ mod dynamic_array;
 #[allow(dead_code)]
 mod dynamic_chunk;
 
+/// Epoch of EventQueue::start_position
+type StartPositionEpoch = crate::utils::Epoch<u32, {i32::MAX as u64}>;
+
 pub mod mpmc{
     //! Multi-producer multi-consumer
     //!
